package RevisableAsset::CMS;

use strict;
use warnings;

sub get_object_asset {
    my ($cb, $app, $obj, $orig_obj) = @_;

    my $q = $app->param;
    my $id = $q->param('id');
    return 1 if !$id;

    # old assets
    my @obj_assets = MT::ObjectAsset->load(
                          {   object_ds => 'entry',
                              object_id => $obj->id
                          } );
    if (@obj_assets) {
        my @orig_asset_ids = map { $_->asset_id } @obj_assets;
        $obj->cache_property( 'assets', undef, \@orig_asset_ids );
    } else {
        $obj->cache_property( 'assets', undef, undef );
    }

    # new assets
    my $include_asset_ids = $q->param('include_asset_ids');
    if ($include_asset_ids) {
        my @asset_ids = split( ',', $include_asset_ids );
        $obj->cache_property( 'object_assets', undef, \@asset_ids );
    } else {
        $obj->cache_property( 'object_assets', undef, undef );
    }
    return 1;
}

sub set_changed_cols {
    my ( $cb, $obj, $orig_obj, $app ) = @_;

    my $q = $app->param;
    my $id = $q->param('id');

    if ($id) {
        my $orig_object_assets = $obj->cache_property( 'assets' );
        my $object_assets = $obj->cache_property( 'object_assets' );
        return 1 if !$orig_object_assets && !$object_assets;

        my @orig_assets;
        if ( 'ARRAY' eq ref($orig_object_assets) ) {
            @orig_assets = sort { $a <=> $b } @$orig_object_assets;
        } else {
            push @orig_assets, $orig_object_assets;
        }

        my @assets;
        if ( 'ARRAY' eq ref($object_assets) ) {
            @assets = sort { $a <=> $b } @$object_assets;
        } else {
            push @assets, $object_assets;
        }

        return 1 if join('', @orig_assets) == join('', @assets);

    } else {
        my $include_asset_ids = $q->param('include_asset_ids');
        return if !$include_asset_ids;
    }

    my $changed_cols = $obj->{changed_revisioned_cols};
    push @$changed_cols, 'assets';
    $obj->{changed_revisioned_cols} = $changed_cols;

    1;

}

sub pack_revision_asset {
    my $cb = shift;
    my ($obj, $values) = @_;

    my $changed_cols = $obj->{changed_revisioned_cols};
    if (grep /^assets$/, @$changed_cols) {

        my $asset_ids = $obj->cache_property( 'object_assets' );
        if (!$asset_ids) {

            # for new entry
            my @obj_assets = MT::ObjectAsset->load(
                                  {   object_ds => 'entry',
                                      object_id => $obj->id
                                  } );
            my @asset_ids;
            foreach my $obj_asset (@obj_assets) {
                push @asset_ids, $obj_asset->asset_id;
            }
            $values->{__rev_assets} = \@asset_ids if @asset_ids;
        } else {
            $values->{__rev_assets} = $asset_ids;
        }
        $values;
    }
}

sub unpack_revision_asset {
    my $cb = shift;
    my ($obj, $packed_obj) = @_;

    if ( my $rev_assets = delete $packed_obj->{__rev_assets} ) {
        $obj->cache_property( 'object_assets', undef, $rev_assets );
    }
}

sub set_entry_param {
    my ($cb, $app, $param, $tmpl) = @_;

    my $q = $app->param;
    my $id = $q->param('id');
    my $type = $q->param('_type');
    my $obj = MT->model($type)->load($id);

    return if !$q->param('r');
    return if $q->param('r') eq $obj->current_revision;

    my $assets;
    my $asset_ids = $obj->cache_property( 'object_assets' );

    foreach my $asset_id (@{$asset_ids}) {
        my $asset = MT::Asset->load($asset_id);
        next if !$asset;

        my $asset_1;
        if ( $asset->class eq 'image' ) {
            $asset_1 = {
                asset_id    => $asset->id,
                asset_name  => $asset->file_name,
                asset_thumb => $asset->thumbnail_url( Width => 100 ),
                asset_type  => $asset->class
            };
        }
        else {
            $asset_1 = {
                asset_id   => $asset->id,
                asset_name => $asset->file_name,
                asset_type => $asset->class
            };
        }
        push @{$assets}, $asset_1;
    }
    $param->{asset_loop} = $assets;
    return 1;
}

1;
